"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.Config = void 0;

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));

var _findIndex = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/find-index"));

var _startsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/starts-with"));

var _values = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/values"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));

var _splice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/splice"));

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));

var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));

var _set = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/set"));

var _from = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/from"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/includes"));

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var fs = _interopRequireWildcard(require("./fs"));

var _plugins = require("./plugins");

var _utils = require("./utils/utils");

var _embarkI18n = require("embark-i18n");

var _embarkUtils = require("embark-utils");

var _configDefaults = require("./configDefaults");

const path = require('path');

const deepEqual = require('deep-equal');

const cloneDeep = require('lodash.clonedeep');

const {
  replaceZeroAddressShorthand
} = _embarkUtils.AddressUtils;

const constants = require('../constants.json');

const DEFAULT_CONFIG_PATH = 'config/';
const embark5ChangesUrl = 'https://framework.embarklabs.io/docs/migrating_from_3.x.html#Updating-to-v5';

class Config {
  constructor(options) {
    this.env = 'default';
    this.blockchainConfig = {};
    this.contractsConfig = {};
    this.pipelineConfig = {};
    this.namesystemConfig = {};
    this.communicationConfig = {};
    this.webServerConfig = void 0;
    this.storageConfig = void 0;
    this.chainTracker = {};
    this.assetFiles = {};
    this.contractsFiles = [];
    this.configDir = void 0;
    this.chainsFile = './chains.json';
    this.plugins = void 0;
    this.logger = void 0;
    this.package = void 0;
    this.events = void 0;
    this.embarkConfig = constants.defaultEmbarkConfig;
    this.context = void 0;
    this.version = void 0;
    this.locale = void 0;
    this.corsParts = [];
    this.providerUrl = '';
    this.contractDirectories = [];
    this.buildDir = void 0;
    this.client = void 0;
    this.dappPath = _embarkUtils.dappPath;
    this.env = options.env || 'default';
    this.webServerConfig = options.webServerConfig;
    this.embarkConfig = options.embarkConfig;
    this.configDir = options.configDir || DEFAULT_CONFIG_PATH;
    this.chainsFile = options.chainsFile;
    this.plugins = options.plugins;
    this.locale = options.locale || 'en';
    this.logger = options.logger;
    this.package = options.package;
    this.events = options.events;
    this.context = options.context || [constants.contexts.any];
    this.version = options.version;
    this.client = options.client;
    this.registerEvents();
  }

  setConfig(configName, newConfig, cb) {
    this[configName] = newConfig;
    cb();
  }

  registerEvents() {
    var _context, _context2, _context3, _context4, _context5;

    this.events.setCommandHandler("config:cors:add", url => {
      this.corsParts.push(url);

      this._updateBlockchainCors();
    });
    this.events.setCommandHandler("config:contractsConfig", cb => {
      cb(null, this.contractsConfig);
    });
    this.events.setCommandHandler("config:storageConfig", cb => {
      cb(null, this.storageConfig);
    });
    this.events.setCommandHandler("config:contractsConfig:set", (0, _bind.default)(_context = this.setConfig).call(_context, this, 'contractsConfig'));
    this.events.setCommandHandler("config:blockchainConfig:set", (0, _bind.default)(_context2 = this.setConfig).call(_context2, this, 'blockchainConfig'));
    this.events.setCommandHandler("config:storageConfig:set", (0, _bind.default)(_context3 = this.setConfig).call(_context3, this, 'storageConfig'));
    this.events.setCommandHandler("config:namesystemConfig:set", (0, _bind.default)(_context4 = this.setConfig).call(_context4, this, 'namesystemConfig'));
    this.events.setCommandHandler("config:communicationConfig:set", (0, _bind.default)(_context5 = this.setConfig).call(_context5, this, 'communicationConfig'));
    this.events.setCommandHandler("config:contractsFiles", cb => {
      cb(null, this.contractsFiles);
    }); // TODO: refactor this so reading the file can be done with a normal resolver or something that takes advantage of the plugin api

    this.events.setCommandHandler("config:contractsFiles:add", (filename, resolver) => {
      resolver = resolver || (callback => {
        callback(fs.readFileSync(filename).toString());
      });

      this.contractsFiles.push(new _embarkUtils.File({
        path: filename,
        originalPath: filename,
        type: _embarkUtils.Types.custom,
        resolver
      }));
    });
    this.events.setCommandHandler("config:contractsFiles:reset", cb => {
      var _context6;

      (0, _forEach.default)(_context6 = this.contractsFiles).call(_context6, file => {
        var _context7, _context8;

        if ((0, _includes.default)(_context7 = file.path).call(_context7, ".embark")) {
          fs.removeSync(file.path);
        }

        this.contractsFiles = (0, _filter.default)(_context8 = this.contractsFiles).call(_context8, contractFile => contractFile.path !== file.path);
      });
      cb();
    });
    this.events.on('file-remove', (fileType, removedPath) => {
      var _context9;

      if (fileType !== 'contract') {
        return;
      }

      const normalizedPath = path.normalize(removedPath);
      this.contractsFiles = (0, _filter.default)(_context9 = this.contractsFiles).call(_context9, file => path.normalize(file.path) !== normalizedPath);
    });
  }

  loadConfigFiles(options) {
    let interceptLogs = options.interceptLogs;

    if (options.interceptLogs === undefined) {
      interceptLogs = true;
    }

    this.embarkConfig.plugins = this.embarkConfig.plugins || {};
    this.plugins = new _plugins.Plugins({
      plugins: this.embarkConfig.plugins,
      logger: this.logger,
      interceptLogs,
      events: this.events,
      config: this,
      context: this.context,
      env: this.env,
      version: this.version,
      client: this.client
    });
    this.loadEmbarkConfigFile();
    this.loadBlockchainConfigFile();
    this.loadStorageConfigFile();
    this.loadContractFiles();
    this.loadCommunicationConfigFile();
    this.loadNameSystemConfigFile();
    this.loadPipelineConfigFile();
    this.loadAssetFiles();
    this.loadContractsConfigFile();
    this.loadExternalContractsFiles();
    this.loadWebServerConfigFile();
    this.loadPluginContractFiles();

    this._updateBlockchainCors();
  }

  reloadConfig() {
    this.loadEmbarkConfigFile();
    this.loadBlockchainConfigFile();
    this.loadStorageConfigFile();
    this.loadContractFiles();
    this.loadCommunicationConfigFile();
    this.loadNameSystemConfigFile();
    this.loadPipelineConfigFile();
    this.loadAssetFiles();
    this.loadContractsConfigFile();
    this.loadExternalContractsFiles();
    this.loadPluginContractFiles();

    this._updateBlockchainCors();
  }

  loadContractFiles() {
    var _context10;

    const loadedContractFiles = this.loadFiles(this.embarkConfig.contracts); // `this.contractsFiles` could've been mutated at runtime using
    // either `config:contractsFiles:add` event or through calls to
    // `loadExternalContractsFiles()`, so we have to make sure we preserve
    // those added files before we reset `this.contractsFiles`.
    //
    // We do that by determining the difference between `loadedContractFiles` and the ones
    // already in memory in `this.contractsFiles`.

    const addedContractFiles = (0, _filter.default)(_context10 = this.contractsFiles).call(_context10, existingFile => !(0, _some.default)(loadedContractFiles).call(loadedContractFiles, file => file.originalPath === existingFile.originalPath));
    this.contractsFiles = (0, _concat.default)(loadedContractFiles).call(loadedContractFiles, addedContractFiles);
  }

  _updateBlockchainCors() {
    const blockchainConfig = this.blockchainConfig;
    const storageConfig = this.storageConfig;
    const webServerConfig = this.webServerConfig;
    let corsParts = cloneDeep(this.corsParts);

    if (blockchainConfig.isDev) {
      corsParts.push('*');
    }

    if (webServerConfig && webServerConfig.host) {
      corsParts.push((0, _embarkUtils.buildUrlFromConfig)(webServerConfig));
    }

    if (storageConfig && storageConfig.enabled) {
      // if getUrl is specified in the config, that needs to be included in cors
      // instead of the concatenated protocol://host:port
      if (storageConfig.upload.getUrl) {
        // remove /ipfs or /bzz: from getUrl if it's there
        let getUrlParts = storageConfig.upload.getUrl.split('/');
        getUrlParts = (0, _slice.default)(getUrlParts).call(getUrlParts, 0, 3);
        const host = (0, _embarkUtils.canonicalHost)(getUrlParts[2].split(':')[0]);
        const port = getUrlParts[2].split(':')[1];
        getUrlParts[2] = port ? [host, port].join(':') : host;
        corsParts.push(getUrlParts.join('/'));
      } else {
        corsParts.push((0, _embarkUtils.buildUrlFromConfig)(storageConfig.upload));
      }
    } // Add cors for the proxy and whisper


    corsParts.push(constants.embarkResourceOrigin);
    corsParts = (0, _from.default)(new _set.default(corsParts));
    this.corsParts = corsParts;
    const cors = corsParts.join(',');

    if (blockchainConfig.rpcCorsDomain === 'auto') {
      blockchainConfig.rpcCorsDomain = cors;
    } else if (typeof blockchainConfig.rpcCorsDomain === 'object') {
      let tempCors = blockchainConfig.rpcCorsDomain.auto ? corsParts : [];
      tempCors = (0, _concat.default)(tempCors).call(tempCors, blockchainConfig.rpcCorsDomain.additionalCors || []);
      blockchainConfig.rpcCorsDomain = tempCors.join(',');
    }

    if (blockchainConfig.wsOrigins === 'auto') {
      blockchainConfig.wsOrigins = cors;
    } else if (typeof blockchainConfig.wsOrigins === 'object') {
      let tempCors = blockchainConfig.wsOrigins.auto ? corsParts : [];
      tempCors = (0, _concat.default)(tempCors).call(tempCors, blockchainConfig.wsOrigins.additionalCors || []);
      blockchainConfig.wsOrigins = tempCors.join(',');
    }
  }

  _loadConfigFile(configFilePath, defaultConfig, enabledByDefault = false) {
    if (!configFilePath) {
      const configToReturn = defaultConfig.default || {};
      configToReturn.enabled = enabledByDefault;
      return configToReturn;
    }

    configFilePath = configFilePath.replace('.json', '').replace('.js', '');
    let config;

    if (fs.existsSync(configFilePath + '.js')) {
      delete require.cache[configFilePath + '.js'];
      config = require(configFilePath + '.js');
    } else if (fs.existsSync(configFilePath + '.json')) {
      config = fs.readJSONSync(configFilePath + '.json');
    } else {
      this.logger.warn((0, _embarkI18n.__)("no config file found at %s using default config", configFilePath));
      return defaultConfig.default || {};
    }

    return config;
  }

  _doMergeConfig(config, defaultConfig, env) {
    const configObject = (0, _embarkUtils.recursiveMerge)(defaultConfig, config);

    if (env) {
      if (env === 'test' && !configObject[env]) {
        // Disabled all configs in tests as they are opt in
        return (0, _assign.default)({}, defaultConfig.default, {
          enabled: false
        });
      }

      return (0, _embarkUtils.recursiveMerge)(configObject.default || {}, configObject[env]);
    } else if (env !== false) {
      this.logger.info((0, _embarkI18n.__)("No environment called %s found. Using defaults.", env));
    }

    return configObject;
  }

  _loadAndMergeConfig(configFilePath, defaultConfig, env, enabledByDefault = false) {
    const config = this._loadConfigFile(configFilePath, defaultConfig, enabledByDefault);

    return this._doMergeConfig(config, defaultConfig, env);
  }

  _getFileOrObject(object, filePath, property) {
    if (typeof object === 'object') {
      return object[property] ? (0, _embarkUtils.dappPath)(object[property]) : object[property];
    }

    return (0, _embarkUtils.dappPath)(object, filePath);
  }
  /*eslint complexity: ["error", 30]*/


  loadBlockchainConfigFile() {
    const blockchainDefaults = (0, _configDefaults.getBlockchainDefaults)(this.env);

    const configFilePath = this._getFileOrObject(this.configDir, 'blockchain', 'blockchain');

    const userConfig = this._loadConfigFile(configFilePath, blockchainDefaults, true);

    const envConfig = userConfig[this.env];

    if (envConfig) {
      if (envConfig.ethereumClientName || envConfig.hasOwnProperty('isDev') || envConfig.hasOwnProperty('mineWhenNeeded')) {
        this.logger.error((0, _embarkI18n.__)('The blockchain config has changed quite a bit in Embark 5\nPlease visit %s to know what has to be changed', embark5ChangesUrl.underline));
        process.exit(1);
      }

      if (envConfig.clientConfig) {
        (0, _assign.default)(envConfig, envConfig.clientConfig);
        delete envConfig.clientConfig;
      }

      switch (envConfig.miningMode) {
        case 'dev':
          envConfig.isDev = true;
          break;

        case 'auto':
          envConfig.isDev = false;
          envConfig.mineWhenNeeded = true;
          break;

        case 'always':
          envConfig.isDev = false;
          envConfig.mineWhenNeeded = false;
          envConfig.mine = true;
          break;

        case 'off':
          envConfig.isDev = false;
          envConfig.mineWhenNeeded = false;
          envConfig.mine = false;
          break;

        default:
          envConfig.isDev = false;
      }

      if (envConfig.cors) {
        var _context11;

        const autoIndex = (0, _indexOf.default)(_context11 = envConfig.cors).call(_context11, 'auto');
        envConfig.rpcCorsDomain = {};
        envConfig.wsOrigins = {};

        if (autoIndex > -1) {
          var _context12;

          envConfig.rpcCorsDomain.auto = true;
          envConfig.wsOrigins.auto = true;
          (0, _splice.default)(_context12 = envConfig.cors).call(_context12, autoIndex, 1);
        } else {
          envConfig.rpcCorsDomain.auto = false;
          envConfig.wsOrigins.auto = false;
        }

        envConfig.rpcCorsDomain.additionalCors = envConfig.cors;
        envConfig.wsOrigins.additionalCors = envConfig.cors;
        delete envConfig.cors;
      }

      userConfig[this.env] = envConfig;
    }

    this.blockchainConfig = this._doMergeConfig(userConfig, blockchainDefaults, this.env);

    if (!configFilePath) {
      this.blockchainConfig.default = true;
    }

    if (this.blockchainConfig.targetGasLimit && this.blockchainConfig.targetGasLimit.toString().match(_embarkUtils.unitRegex)) {
      this.blockchainConfig.targetGasLimit = (0, _embarkUtils.getWeiBalanceFromString)(this.blockchainConfig.targetGasLimit);
    }

    if (this.blockchainConfig.gasPrice && this.blockchainConfig.gasPrice.toString().match(_embarkUtils.unitRegex)) {
      this.blockchainConfig.gasPrice = (0, _embarkUtils.getWeiBalanceFromString)(this.blockchainConfig.gasPrice);
    }

    if (this.blockchainConfig.accounts) {
      var _context13;

      (0, _forEach.default)(_context13 = this.blockchainConfig.accounts).call(_context13, acc => {
        if (acc.balance && acc.balance.toString().match(_embarkUtils.unitRegex)) {
          acc.balance = (0, _embarkUtils.getWeiBalanceFromString)(acc.balance);
        }
      });
    }

    if (this.blockchainConfig.endpoint) {
      const {
        type,
        host,
        port
      } = (0, _embarkUtils.deconstructUrl)(this.blockchainConfig.endpoint);

      if (type === 'ws') {
        this.blockchainConfig.wsHost = host;
        this.blockchainConfig.wsPort = port;
        this.blockchainConfig.wsRPC = true;
      } else {
        this.blockchainConfig.rpcHost = host;
        this.blockchainConfig.rpcPort = port;
        this.blockchainConfig.wsRPC = false;
      }
    } else {
      const urlConfig = this.blockchainConfig.wsHost ? {
        host: this.blockchainConfig.wsHost,
        port: this.blockchainConfig.wsPort,
        type: 'ws'
      } : {
        host: this.blockchainConfig.rpcHost,
        port: this.blockchainConfig.rpcPort,
        type: 'rpc'
      };
      this.blockchainConfig.endpoint = (0, _embarkUtils.buildUrlFromConfig)(urlConfig);
      this.blockchainConfig.isAutoEndpoint = true;
    }

    const accountDocsMessage = (0, _embarkI18n.__)('For more info, check the docs: %s', 'https://framework.embarklabs.io/docs/blockchain_accounts_configuration.html'.underline);

    if (this.blockchainConfig.account) {
      this.logger.error((0, _embarkI18n.__)('The `account` config for the blockchain was removed. Please use `accounts` instead.'));
      this.logger.error(accountDocsMessage);
      process.exit(1);
    }

    if (this.blockchainConfig.simulatorMnemonic) {
      this.logger.error((0, _embarkI18n.__)('The `simulatorMnemonic` config for the blockchain was removed. Please use `accounts` instead.'));
      this.logger.error(accountDocsMessage);
      process.exit(1);
    }

    this.events.emit('config:load:blockchain', this.blockchainConfig);
  }

  loadContractsConfigFile() {
    let configObject = (0, _configDefaults.getContractDefaults)(this.embarkConfig.versions);
    const contractsConfigs = this.plugins.getPluginsProperty('contractsConfig', 'contractsConfigs');
    (0, _forEach.default)(contractsConfigs).call(contractsConfigs, pluginConfig => {
      configObject = (0, _embarkUtils.recursiveMerge)(configObject, pluginConfig);
    });

    const configFilePath = this._getFileOrObject(this.configDir, 'contracts', 'contracts');

    let newContractsConfig = this._loadAndMergeConfig(configFilePath, configObject, this.env);

    if (newContractsConfig.contracts) {
      this.logger.error((0, _embarkI18n.__)('`contracts` has been renamed `deploy` in contracts config\nFor more information: %s', embark5ChangesUrl.underline));
      process.exit(1);
    }

    if (newContractsConfig.deployment) {
      this.logger.error((0, _embarkI18n.__)('`deployment` has been removed from contracts config and is now part of blockchain config\nFor more information: %s', embark5ChangesUrl.underline));
      process.exit(1);
    }

    if (newContractsConfig.gas.match(_embarkUtils.unitRegex)) {
      newContractsConfig.gas = (0, _embarkUtils.getWeiBalanceFromString)(newContractsConfig.gas);
    }

    newContractsConfig = (0, _embarkUtils.prepareContractsConfig)(newContractsConfig);
    const afterDeploy = newContractsConfig.afterDeploy;

    if ((0, _isArray.default)(afterDeploy)) {
      newContractsConfig.afterDeploy = (0, _map.default)(afterDeploy).call(afterDeploy, replaceZeroAddressShorthand);
    }

    if (!deepEqual(newContractsConfig, this.contractsConfig)) {
      this.contractsConfig = newContractsConfig;
    }

    this.events.emit('config:load:contracts', this.contractsConfig);
  }

  loadExternalContractsFiles() {
    const contracts = this.contractsConfig.contracts;
    const storageConfig = this.storageConfig;

    if (storageConfig && storageConfig.upload && storageConfig.upload.getUrl) {
      this.providerUrl = storageConfig.upload.getUrl;
    }

    for (const contract of (0, _values.default)(contracts)) {
      var _context14, _context15, _context16, _context17;

      if (!contract.file) {
        continue;
      }

      let externalContractFile;

      if ((0, _startsWith.default)(_context14 = contract.file).call(_context14, 'http') || (0, _startsWith.default)(_context15 = contract.file).call(_context15, 'git') || (0, _startsWith.default)(_context16 = contract.file).call(_context16, 'ipfs') || (0, _startsWith.default)(_context17 = contract.file).call(_context17, 'bzz')) {
        const fileObj = (0, _embarkUtils.getExternalContractUrl)(contract.file, this.providerUrl);

        if (!fileObj) {
          return this.logger.error((0, _embarkI18n.__)("HTTP contract file not found") + ": " + contract.file);
        }

        externalContractFile = new _embarkUtils.File({
          path: fileObj.filePath,
          originalPath: fileObj.filePath,
          type: _embarkUtils.Types.http,
          basedir: '',
          externalUrl: fileObj.url,
          storageConfig
        });
      } else if (fs.existsSync(contract.file)) {
        externalContractFile = new _embarkUtils.File({
          path: contract.file,
          originalPath: contract.file,
          type: _embarkUtils.Types.dappFile,
          basedir: '',
          storageConfig
        });
      } else if (fs.existsSync(path.join('./node_modules/', contract.file))) {
        const completePath = path.join('./node_modules/', contract.file);
        externalContractFile = new _embarkUtils.File({
          path: completePath,
          originalPath: completePath,
          type: _embarkUtils.Types.dappFile,
          basedir: '',
          storageConfig
        });
      }

      if (externalContractFile) {
        var _context18;

        const index = (0, _findIndex.default)(_context18 = this.contractsFiles).call(_context18, contractFile => contractFile.originalPath === externalContractFile.originalPath); // It's important that we only add `externalContractFile` if it doesn't exist already
        // within `contractsFiles`, otherwise we keep adding duplicates in subsequent
        // compilation routines creating a memory leak.

        if (index > -1) {
          this.contractsFiles[index] = externalContractFile;
        } else {
          this.contractsFiles.push(externalContractFile);
        }
      } else {
        this.logger.error((0, _embarkI18n.__)("contract file not found") + ": " + contract.file);
      }
    }
  }

  loadStorageConfigFile() {
    const configObject = {
      default: {
        enabled: true,
        available_providers: ["ipfs", "swarm"],
        ipfs_bin: "ipfs",
        upload: {
          provider: "ipfs",
          protocol: "http",
          host: _embarkUtils.defaultHost,
          port: 5001,
          getUrl: "http://localhost:8080/ipfs/"
        },
        dappConnection: [{
          provider: "ipfs",
          host: "localhost",
          port: 5001,
          getUrl: "http://localhost:8080/ipfs/"
        }]
      }
    };

    const configFilePath = this._getFileOrObject(this.configDir, 'storage', 'storage');

    this.storageConfig = this._loadAndMergeConfig(configFilePath, configObject, this.env);
    this.events.emit('config:load:storage', this.storageConfig);
  }

  loadNameSystemConfigFile() {
    // todo: spec out names for registration in the file itself for a dev chain
    const configObject = {
      default: {
        enabled: false
      }
    };

    const configFilePath = this._getFileOrObject(this.configDir, 'namesystem', 'namesystem');

    this.namesystemConfig = this._loadAndMergeConfig(configFilePath, configObject, this.env);
  }

  loadCommunicationConfigFile() {
    const configObject = {
      default: {
        enabled: false,
        provider: "whisper",
        available_providers: ["whisper"],
        client: "geth",
        connection: {
          host: _embarkUtils.defaultHost,
          port: 8547,
          type: "ws"
        }
      }
    };

    const configFilePath = this._getFileOrObject(this.configDir, 'communication', 'communication');

    this.communicationConfig = this._loadAndMergeConfig(configFilePath, configObject, this.env);
    this.events.emit('config:load:communication', this.communicationConfig);
  }

  loadWebServerConfigFile() {
    const configObject = {
      enabled: true,
      host: _embarkUtils.defaultHost,
      openBrowser: true,
      port: 8000,
      enableCatchAll: true,
      protocol: "http"
    };

    const configFilePath = this._getFileOrObject(this.configDir, 'webserver', 'webserver');

    const webServerConfig = this._loadAndMergeConfig(configFilePath, configObject, false);

    if (webServerConfig.https) {
      try {
        webServerConfig.certOptions = {
          key: fs.readFileSync(webServerConfig.key),
          cert: fs.readFileSync(webServerConfig.cert)
        };
        webServerConfig.protocol = 'https';
      } catch (e) {
        this.logger.error(e.message);
        this.logger.warn('Invalid path for key/cert in config/webserver.js. Using http instead.');
        webServerConfig.certOptions = {};
        webServerConfig.protocol = 'http';
      }
    }

    if (configFilePath === false) {
      this.webServerConfig = {
        enabled: false
      };
      return;
    }

    if (this.webServerConfig) {
      // cli flags to `embark run` should override configFile and defaults (configObject)
      this.webServerConfig = (0, _embarkUtils.recursiveMerge)(webServerConfig, this.webServerConfig);
    } else {
      this.webServerConfig = webServerConfig;
    }

    if (!this.pipelineConfig.enabled) {
      this.webServerConfig.enabled = false;
    }

    this.events.emit('config:load:webserver', this.webServerConfig);
  }

  loadEmbarkConfigFile() {
    var _context19;

    this.embarkConfig = (0, _embarkUtils.recursiveMerge)(constants.defaultEmbarkConfig, this.embarkConfig);
    const contracts = this.embarkConfig.contracts; // determine contract 'root' directories

    this.contractDirectories = (0, _map.default)(_context19 = (0, _map.default)(contracts).call(contracts, dir => {
      return dir.split("**")[0];
    })).call(_context19, dir => {
      return dir.split("*.")[0];
    });
    this.contractDirectories.push(constants.httpContractsDirectory);
    this.buildDir = this.embarkConfig.buildDir;
    this.configDir = this.embarkConfig.config;
  }

  loadPipelineConfigFile() {
    const defaultPipelineConfig = {
      typescript: false,
      enabled: true
    };

    let pipelineConfigPath = this._getFileOrObject(this.configDir, 'pipeline', 'pipeline'); // Embark applications in "simple" mode that aren't aware of `pipeline.js` configuration capabilities
    // won't have a pipeline config path so we need to perform this safety check here, otherwise the
    // next expression is going to throw.


    if (pipelineConfigPath !== undefined) {
      // At this point, `pipelineConfigPath` could be either `config/pipeline` or a filepath including its extension.
      // We need to make sure that we always have an extension.
      pipelineConfigPath = `${(0, _embarkUtils.dappPath)(pipelineConfigPath)}${path.extname(pipelineConfigPath) === '.js' ? '' : '.js'}`;
    }

    let pipelineConfig = defaultPipelineConfig;

    if (pipelineConfigPath && fs.existsSync(pipelineConfigPath)) {
      delete require.cache[pipelineConfigPath];
      pipelineConfig = (0, _embarkUtils.recursiveMerge)((0, _embarkUtils.recursiveMerge)(true, pipelineConfig), require(pipelineConfigPath));
    }

    this.pipelineConfig = pipelineConfig;
    this.events.emit('config:load:pipeline', this.pipelineConfig);
  }

  loadAssetFiles() {
    var _context20;

    if (!this.embarkConfig.app) {
      return;
    }

    (0, _forEach.default)(_context20 = (0, _keys.default)(this.embarkConfig.app)).call(_context20, targetFile => {
      this.assetFiles[targetFile] = this.loadFiles(this.embarkConfig.app[targetFile]);
    });
  }

  loadFiles(files) {
    var _context21;

    const self = this;
    const originalFiles = (0, _utils.filesMatchingPattern)(files);
    const readFiles = [];
    const storageConfig = self.storageConfig;
    (0, _filter.default)(_context21 = (0, _filter.default)(originalFiles).call(originalFiles, file => {
      return file[0] === '$' || (0, _indexOf.default)(file).call(file, '.') >= 0;
    })).call(_context21, file => {
      const basedir = findMatchingExpression(file, files);
      readFiles.push(new _embarkUtils.File({
        path: file,
        originalPath: file,
        type: _embarkUtils.Types.dappFile,
        basedir,
        storageConfig
      }));
    });
    const filesFromPlugins = [];
    const filePlugins = self.plugins.getPluginsFor('pipelineFiles');
    (0, _forEach.default)(filePlugins).call(filePlugins, plugin => {
      try {
        const fileObjects = plugin.runFilePipeline();

        for (const fileObject of fileObjects) {
          filesFromPlugins.push(fileObject);
        }
      } catch (err) {
        self.logger.error(err.message);
      }
    });
    (0, _filter.default)(filesFromPlugins).call(filesFromPlugins, file => {
      if (file.intendedPath && (0, _utils.fileMatchesPattern)(files, file.intendedPath) || (0, _utils.fileMatchesPattern)(files, file.file)) {
        readFiles.push(file);
      }
    });
    return readFiles;
  } // NOTE: this doesn't work for internal modules


  loadPluginContractFiles() {
    const self = this;
    const storageConfig = self.storageConfig;
    const contractsPlugins = this.plugins.getPluginsFor('contractFiles');
    (0, _forEach.default)(contractsPlugins).call(contractsPlugins, plugin => {
      var _context22;

      (0, _forEach.default)(_context22 = plugin.contractsFiles).call(_context22, file => {
        const filename = file.replace('./', '');
        self.contractsFiles.push(new _embarkUtils.File({
          path: filename,
          originalPath: path.join(plugin.pluginPath, filename),
          pluginPath: plugin.pluginPath,
          type: _embarkUtils.Types.custom,
          storageConfig,

          resolver(callback) {
            callback(plugin.loadPluginFile(file));
          }

        }));
      });
    });
  }

}

exports.Config = Config;

function findMatchingExpression(filename, filesExpressions) {
  for (const fileExpression of filesExpressions) {
    const matchingFiles = (0, _utils.filesMatchingPattern)(fileExpression);

    for (const matchFile of matchingFiles) {
      if (matchFile === filename) {
        return path.dirname(fileExpression).replace(/\*/g, '');
      }
    }
  }

  return path.dirname(filename);
}
//# sourceMappingURL=config.js.map