"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.Plugin = void 0;

var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _set = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/set"));

var _from = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/from"));

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));

var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/includes"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));

var _utils = require("./utils/utils");

var _embarkI18n = require("embark-i18n");

var _embarkUtils = require("embark-utils");

var fs = _interopRequireWildcard(require("fs-extra"));

var _path = require("path");

const deepEqual = require('deep-equal');

const constants = (0, fs.readJsonSync)((0, _path.join)(__dirname, '../constants.json')); // Default priority of actions with no specified priority. 1 => Highest

const DEFAULT_ACTION_PRIORITY = 50; // TODO: pass other params like blockchainConfig, contract files, etc..

class Plugin {
  constructor(options) {
    this.dappPath = _embarkUtils.dappPath;
    this.embarkPath = _embarkUtils.embarkPath;
    this.name = void 0;
    this.isInternal = void 0;
    this.pluginModule = void 0;
    this.pluginPath = void 0;
    this.pluginConfig = void 0;
    this.shouldInterceptLogs = void 0;
    this.clientWeb3Providers = [];
    this.beforeDeploy = [];
    this.contractsGenerators = [];
    this.generateCustomContractCode = null;
    this.testContractFactory = null;
    this.pipeline = [];
    this.pipelineFiles = [];
    this.console = [];
    this.contractsConfigs = [];
    this.contractsFiles = [];
    this.compilers = [];
    this.serviceChecks = [];
    this.dappGenerators = [];
    this.pluginTypes = [];
    this.uploadCmds = [];
    this.apiCalls = [];
    this.imports = [];
    this.embarkjs_code = [];
    this.generated_code = [];
    this.embarkjs_init_code = [];
    this.embarkjs_init_console_code = [];
    this.fs = void 0;
    this.afterContractsDeployActions = [];
    this.onDeployActions = [];
    this.eventActions = {};
    this._loggerObject = void 0;
    this.events = void 0;
    this.config = void 0;
    this.plugins = void 0;
    this.pluginsAPI = void 0;
    this.env = void 0;
    this.loaded = false;
    this.currentContext = void 0;
    this.acceptedContext = void 0;
    this.version = void 0;
    this.constants = void 0;
    this.logger = void 0;
    this.name = options.name;
    this.isInternal = options.isInternal;
    this.pluginModule = options.pluginModule;
    this.pluginPath = options.pluginPath;
    this.pluginConfig = options.pluginConfig;
    this.shouldInterceptLogs = options.interceptLogs;
    this.fs = fs;
    this._loggerObject = options.logger;
    this.logger = this._loggerObject; // Might get changed if we do intercept

    this.events = options.events;
    this.config = options.config;
    this.plugins = options.plugins;
    this.env = options.env;
    this.currentContext = options.context;
    this.acceptedContext = options.pluginConfig.context || [constants.contexts.any];
    this.version = options.version;
    this.constants = constants;
    this.pluginsAPI = options.pluginsAPI;

    if (!(0, _isArray.default)(this.currentContext)) {
      this.currentContext = [this.currentContext];
    }

    if (!(0, _isArray.default)(this.acceptedContext)) {
      this.acceptedContext = [this.acceptedContext];
    }

    this.registerActionForEvent("tests:config:updated", {
      priority: 30
    }, async ({
      accounts
    }, cb) => {
      this.config.blockchainConfig.accounts = accounts;
      cb(null, null);
    });
  }

  _log(type) {
    this._loggerObject[type](this.name + ':', ...(0, _slice.default)([]).call(arguments, 1));
  }

  isContextValid() {
    var _context, _context2, _context3;

    if ((0, _includes.default)(_context = this.currentContext).call(_context, constants.contexts.any) || (0, _includes.default)(_context2 = this.acceptedContext).call(_context2, constants.contexts.any)) {
      return true;
    }

    return (0, _some.default)(_context3 = this.acceptedContext).call(_context3, context => {
      var _context4;

      return (0, _includes.default)(_context4 = this.currentContext).call(_context4, context);
    });
  }

  hasContext(context) {
    var _context5;

    return (0, _includes.default)(_context5 = this.currentContext).call(_context5, context);
  }

  loadPlugin() {
    if (!this.isContextValid()) {
      this.logger.warn((0, _embarkI18n.__)('Plugin {{name}} can only be loaded in the context of "{{contexts}}"', {
        name: this.name,
        contexts: this.acceptedContext.join(', ')
      }));
      return false;
    }

    this.loaded = true;

    if ((0, _embarkUtils.isEs6Module)(this.pluginModule)) {
      if (this.pluginModule.default) {
        this.pluginModule = this.pluginModule.default;
      }

      return new this.pluginModule(this);
    }

    this.pluginModule.call(this, this);
  }

  loadInternalPlugin() {
    if ((0, _embarkUtils.isEs6Module)(this.pluginModule)) {
      if (this.pluginModule.default) {
        this.pluginModule = this.pluginModule.default;
      }
    }

    return new this.pluginModule(this, this.pluginConfig);
    /*eslint no-new: "off"*/
  }

  loadPluginFile(filename) {
    return (0, fs.readFileSync)(this.pathToFile(filename)).toString();
  }

  pathToFile(filename) {
    if (!this.pluginPath) {
      throw new Error('pluginPath not defined for plugin: ' + this.name);
    }

    return (0, _embarkUtils.joinPath)(this.pluginPath, filename);
  } // TODO: add deploy provider


  registerClientWeb3Provider(cb) {
    this.clientWeb3Providers.push(cb);
    this.addPluginType('clientWeb3Provider');
  }

  registerContractsGeneration(cb) {
    this.contractsGenerators.push(cb);
    this.addPluginType('contractGeneration');
  }

  registerCustomContractGenerator(cb) {
    this.generateCustomContractCode = cb;
    this.addPluginType('customContractGeneration');
  }

  registerTestContractFactory(cb) {
    this.testContractFactory = cb;
    this.addPluginType('testContractFactory');
  }

  registerPipeline(matcthingFiles, cb) {
    // TODO: generate error for more than one pipeline per plugin
    this.pipeline.push({
      matcthingFiles,
      cb
    });
    this.addPluginType('pipeline');
  }

  registerDappGenerator(framework, cb) {
    this.dappGenerators.push({
      framework,
      cb
    });
    this.pluginTypes.push('dappGenerator');
  }

  registerCustomType(type) {
    this.pluginTypes.push(type);
  }

  addFileToPipeline(file, intendedPath, options) {
    this.pipelineFiles.push({
      file,
      intendedPath,
      options
    });
    this.addPluginType('pipelineFiles');
  }

  addContractFile(file) {
    if (this.isInternal) {
      throw new Error("this API cannot work for internal modules. please use an event command instead: config:contractsFiles:add");
    }

    this.contractsFiles.push(file);
    this.addPluginType('contractFiles');
  }

  registerConsoleCommand(optionsOrCb) {
    if (typeof optionsOrCb === 'function') {
      this.logger.warn((0, _embarkI18n.__)('Registering console commands with function syntax is deprecated and will likely be removed in future versions of Embark'));
      this.logger.info((0, _embarkI18n.__)('You can find the new API documentation here: %s', 'https://framework.embarklabs.io/docs/plugin_reference.html#registerConsoleCommand-options'.underline));
    }

    this.console.push(optionsOrCb);
    this.addPluginType('console');
  } // TODO: this only works for services done on startup


  registerServiceCheck(checkName, checkFn, time) {
    this.serviceChecks.push({
      checkName,
      checkFn,
      time
    });
    this.addPluginType('serviceChecks');
  }

  has(pluginType) {
    var _context6;

    return (0, _indexOf.default)(_context6 = this.pluginTypes).call(_context6, pluginType) >= 0;
  }

  addPluginType(pluginType) {
    this.pluginTypes.push(pluginType);
    this.pluginTypes = (0, _from.default)(new _set.default(this.pluginTypes));
  }

  generateProvider(args) {
    var _context7;

    return (0, _map.default)(_context7 = this.clientWeb3Providers).call(_context7, function (cb) {
      return cb.call(this, args);
    }).join("\n");
  }

  generateContracts(args) {
    var _context8;

    return (0, _map.default)(_context8 = this.contractsGenerators).call(_context8, function (cb) {
      return cb.call(this, args);
    }).join("\n");
  }

  registerContractConfiguration(config) {
    this.contractsConfigs.push(config);
    this.addPluginType('contractsConfig');
  }

  registerCompiler(extension, cb) {
    this.compilers.push({
      extension,
      cb
    });
    this.addPluginType('compilers');
  }

  registerUploadCommand(cmd, cb) {
    this.uploadCmds.push({
      cmd,
      cb
    });
    this.addPluginType('uploadCmds');
  }

  addCodeToEmbarkJS(code) {
    var _context9;

    this.addPluginType('embarkjsCode'); // TODO: what is this/why

    if (!(0, _some.default)(_context9 = this.embarkjs_code).call(_context9, existingCode => deepEqual(existingCode, code))) {
      this.embarkjs_code.push(code);
    }
  }

  addGeneratedCode(codeCb) {
    this.addPluginType('generatedCode');
    this.generated_code.push(codeCb);
  }

  addProviderInit(providerType, code, initCondition) {
    this.embarkjs_init_code[providerType] = this.embarkjs_init_code[providerType] || [];
    this.embarkjs_init_code[providerType].push([code, initCondition]);
    this.addPluginType('initCode');
  }

  addConsoleProviderInit(providerType, code, initCondition) {
    var _context10;

    this.embarkjs_init_console_code[providerType] = this.embarkjs_init_console_code[providerType] || [];
    this.addPluginType('initConsoleCode');
    const toAdd = [code, initCondition];

    if (!(0, _some.default)(_context10 = this.embarkjs_init_console_code[providerType]).call(_context10, initConsoleCode => deepEqual(initConsoleCode, toAdd))) {
      this.embarkjs_init_console_code[providerType].push(toAdd);
    }
  }

  registerImportFile(importName, importLocation) {
    this.imports.push([importName, importLocation]);
    this.addPluginType('imports');
  }

  registerActionForEvent(eventName, options, cb) {
    if (typeof options === 'function') {
      cb = options;
      options = {};
    }

    if (!this.eventActions[eventName]) {
      this.eventActions[eventName] = [];
    }

    this.eventActions[eventName].push({
      action: cb,
      options: (0, _assign.default)({
        priority: DEFAULT_ACTION_PRIORITY
      }, options)
    });
    this.addPluginType('eventActions');
  }

  registerAPICall(method, endpoint, cb) {
    this.apiCalls.push({
      method,
      endpoint,
      cb
    });
    this.addPluginType('apiCalls');
    this.events.emit('plugins:register:api', {
      method,
      endpoint,
      cb
    });
  }

  runFilePipeline() {
    var _context11;

    return (0, _map.default)(_context11 = this.pipelineFiles).call(_context11, file => {
      const obj = {};
      obj.filename = file.file.replace('./', '');
      obj.content = this.loadPluginFile(file.file).toString();
      obj.intendedPath = file.intendedPath;
      obj.options = file.options;
      obj.path = this.pathToFile(obj.filename);
      return obj;
    });
  }

  runPipeline(args) {
    // TODO: should iterate the pipelines
    const pipeline = this.pipeline[0];
    const shouldRunPipeline = (0, _utils.fileMatchesPattern)(pipeline.matcthingFiles, args.targetFile);

    if (shouldRunPipeline) {
      return pipeline.cb.call(this, args);
    }

    return args.source;
  }

}

exports.Plugin = Plugin;
//# sourceMappingURL=plugin.js.map