"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.Plugins = void 0;

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _sort = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/sort"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/reduce"));

var _entries = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/entries"));

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var fs = _interopRequireWildcard(require("./fs"));

var _plugin = require("./plugin");

var async = _interopRequireWildcard(require("async"));

var _embarkUtils = require("embark-utils");

var _findUp = _interopRequireDefault(require("find-up"));

var _path = require("path");

class Plugins {
  constructor(options) {
    this.pluginList = [];
    this.interceptLogs = void 0;
    this.plugins = [];
    this.logger = void 0;
    this.events = void 0;
    this.config = void 0;
    this.context = void 0;
    this.fs = void 0;
    this.env = void 0;
    this.version = void 0;
    this.client = void 0;
    this.pluginList = options.plugins || [];
    this.interceptLogs = options.interceptLogs; // TODO: need backup 'NullLogger'

    this.logger = options.logger;
    this.events = options.events;
    this.config = options.config;
    this.context = options.context;
    this.fs = fs;
    this.env = options.env;
    this.version = options.version;
    this.client = options.client;
  }

  loadPlugins() {
    var _context, _context2;

    (0, _forEach.default)(_context = (0, _entries.default)(Plugins.deprecated)).call(_context, ([pluginName, embarkVersion]) => {
      if (this.pluginList[pluginName]) {
        delete this.pluginList[pluginName];
        this.logger.warn(`${pluginName} plugin was not loaded because it has been deprecated as of embark v${embarkVersion}, please remove it from this project's embark.json and package.json`);
      }
    });
    (0, _forEach.default)(_context2 = (0, _entries.default)(this.pluginList)).call(_context2, ([pluginName, pluginConfig]) => {
      this.loadPlugin(pluginName, pluginConfig);
    });
  }

  listPlugins() {
    var _context3;

    return (0, _reduce.default)(_context3 = this.plugins).call(_context3, (list, plugin) => {
      if (plugin.loaded) {
        list.push(plugin.name);
      }

      return list;
    }, []);
  } // for services that act as a plugin but have core functionality


  createPlugin(pluginName, pluginConfig) {
    const plugin = {};
    const pluginPath = false;
    const pluginWrapper = new _plugin.Plugin({
      name: pluginName,
      pluginModule: plugin,
      pluginConfig,
      logger: this.logger,
      pluginPath,
      interceptLogs: this.interceptLogs,
      events: this.events,
      config: this.config,
      plugins: this.plugins,
      pluginsAPI: this,
      fs: this.fs,
      isInternal: true,
      context: this.context,
      client: this.client
    });
    this.plugins.push(pluginWrapper);
    return pluginWrapper;
  }

  loadInternalPlugin(pluginName, pluginConfig, isPackage) {
    let pluginPath;
    let plugin;

    if (isPackage) {
      pluginPath = pluginName;
      plugin = require(pluginName);
    } else {
      pluginPath = (0, _embarkUtils.embarkPath)('dist/lib/modules/' + pluginName);
      plugin = require(pluginPath);
    }

    if (plugin.default) {
      plugin = plugin.default;
    }

    const pluginWrapper = new _plugin.Plugin({
      name: pluginName,
      pluginModule: plugin,
      pluginConfig: pluginConfig || {},
      logger: this.logger,
      pluginPath,
      interceptLogs: this.interceptLogs,
      events: this.events,
      config: this.config,
      plugins: this.plugins,
      pluginsAPI: this,
      fs: this.fs,
      isInternal: true,
      context: this.context,
      env: this.env,
      client: this.client
    });
    const pluginInstance = pluginWrapper.loadInternalPlugin();
    this.plugins.push(pluginWrapper);
    return pluginInstance;
  }

  loadPlugin(pluginName, pluginConfig) {
    const pluginPath = (0, _path.dirname)(_findUp.default.sync('package.json', {
      cwd: (0, _path.dirname)(require.resolve(pluginName, {
        paths: [(0, _embarkUtils.dappPath)()]
      }))
    }));

    let plugin = require(pluginPath);

    if (plugin.default) {
      plugin = plugin.default;
    }

    const pluginWrapper = new _plugin.Plugin({
      name: pluginName,
      pluginModule: plugin,
      pluginConfig,
      logger: this.logger,
      pluginPath,
      interceptLogs: this.interceptLogs,
      events: this.events,
      config: this.config,
      plugins: this.plugins,
      pluginsAPI: this,
      fs: this.fs,
      isInternal: false,
      context: this.context,
      version: this.version,
      client: this.client
    });
    pluginWrapper.loadPlugin();
    this.plugins.push(pluginWrapper);
  }

  getPluginsFor(pluginType) {
    var _context4;

    return (0, _filter.default)(_context4 = this.plugins).call(_context4, plugin => {
      return plugin.has(pluginType);
    });
  }

  getPluginsProperty(pluginType, property, sub_property) {
    var _context5;

    const matchingPlugins = (0, _filter.default)(_context5 = this.plugins).call(_context5, plugin => {
      return plugin.has(pluginType);
    }); // Sort internal plugins first

    (0, _sort.default)(matchingPlugins).call(matchingPlugins, (a, b) => {
      if (a.isInternal) {
        return -1;
      }

      if (b.isInternal) {
        return 1;
      }

      return 0;
    });
    let matchingProperties = (0, _map.default)(matchingPlugins).call(matchingPlugins, plugin => {
      if (sub_property) {
        return plugin[property][sub_property];
      }

      return plugin[property];
    }); // Remove empty properties

    matchingProperties = (0, _filter.default)(matchingProperties).call(matchingProperties, prop => prop); // return flattened list

    if (matchingProperties.length === 0) {
      return [];
    }

    return (0, _reduce.default)(matchingProperties).call(matchingProperties, (a, b) => (0, _concat.default)(a).call(a, b)) || [];
  }

  getPluginsPropertyAndPluginName(pluginType, property, sub_property) {
    var _context6;

    const matchingPlugins = (0, _filter.default)(_context6 = this.plugins).call(_context6, plugin => {
      return plugin.has(pluginType);
    }); // Sort internal plugins first

    (0, _sort.default)(matchingPlugins).call(matchingPlugins, (a, b) => {
      if (a.isInternal) {
        return -1;
      }

      if (b.isInternal) {
        return 1;
      }

      return 0;
    });
    let matchingProperties = [];
    (0, _forEach.default)(matchingPlugins).call(matchingPlugins, plugin => {
      if (sub_property) {
        for (const kall of plugin[property][sub_property] || []) {
          matchingProperties.push([kall, plugin.name]);
        }

        return;
      }

      for (const kall of plugin[property] || []) {
        matchingProperties.push([kall, plugin.name]);
      }
    }); // Remove empty properties

    matchingProperties = (0, _filter.default)(matchingProperties).call(matchingProperties, prop => prop[0]); // return flattened list

    if (matchingProperties.length === 0) {
      return [];
    } // return matchingProperties.reduce((a,b) => { return a.concat(b); }) || [];


    return matchingProperties;
  } // TODO: because this is potentially hanging, we should issue a trace warning if the event does not exists


  runActionsForEvent(eventName, args, cb) {
    const self = this;

    if (typeof args === 'function') {
      cb = args;
      args = [];
    }

    const actionPlugins = this.getPluginsPropertyAndPluginName('eventActions', 'eventActions', eventName);

    if (actionPlugins.length === 0) {
      return cb(null, args);
    }

    (0, _sort.default)(actionPlugins).call(actionPlugins, (a, b) => {
      const aPriority = a[0].options.priority;
      const bPriority = b[0].options.priority;

      if (aPriority < bPriority) {
        return -1;
      }

      if (aPriority > bPriority) {
        return 1;
      }

      return 0;
    });
    this.events.log("ACTION", eventName, "");
    async.reduce(actionPlugins, args, (current_args, pluginObj, nextEach) => {
      const [plugin, pluginName] = pluginObj;
      self.events.log("== ACTION FOR " + eventName, plugin.action.name, pluginName);

      if (typeof args === 'function') {
        plugin.action.call(plugin.action, (...params) => {
          nextEach(...(params || current_args));
        });
      } else {
        plugin.action.call(plugin.action, args, (...params) => {
          nextEach(...(params || current_args));
        });
      }
    }, cb);
  }

  emitAndRunActionsForEvent(eventName, args, cb) {
    if (typeof args === 'function') {
      cb = args;
      args = [];
    }

    this.events.emit(eventName, args);
    return this.runActionsForEvent(eventName, args, cb);
  }

}

exports.Plugins = Plugins;
Plugins.deprecated = {
  'embarkjs-connector-web3': '4.1.0'
};
//# sourceMappingURL=plugins.js.map