export class ProcessLauncher {
    /**
     * Constructor of ProcessLauncher. Forks the module and sets up the message handling
     * @param {Object}    options   Options tp start the process
     *        * modulePath      {String}    Absolute path to the module to fork
     *        * logger          {Object}    Logger
     *        * events          {Function}  Events Emitter instance
     * @return {ProcessLauncher}    The ProcessLauncher instance
     */
    constructor(options: any);
    name: any;
    process: any;
    logger: any;
    events: any;
    silent: any;
    exitCallback: any;
    embark: any;
    logs: any[];
    subscriptions: {};
    setSilent(value: any): void;
    _subscribeToMessages(): void;
    _handleEvent(msg: any): void;
    _checkSubscriptions(msg: any): void;
    /**
     * Subscribe to a message using a key-value pair
     * @param {String}    key       Message key to subscribe to
     * @param {String}    value     Value that the above key must have for the callback to be called
     * @param {Function}  callback  callback(response)
     * @return {void}
     */
    on(key: string, value: string, callback: Function): void;
    /**
     * Same as .on, but only triggers once
     * @param {String}    key       Message key to subscribe to
     * @param {String}    value     Value that the above key must have for the callback to be called
     * @param {Function}  callback  callback(response)
     * @return {void}
     */
    once(key: string, value: string, callback: Function): void;
    /**
     * Unsubscribes from a previously subscribed key-value pair (or key if no value)
     * @param {String}  key     Message key to unsubscribe
     * @param {String}  value   [Optional] Value of the key to unsubscribe
     *                            If there is no value, unsubscribes from all the values of that key
     * @return {void}
     */
    unsubscribeTo(key: string, value: string): void;
    /**
     * Unsubscribes from all subscriptions
     * @return {void}
     */
    unsubscribeToAll(): void;
    /**
     * Sends a message to the child process. Same as ChildProcess.send()
     * @params {Object}   message     Message to send
     * For other parameters, see:
     *  https://nodejs.org/api/child_process.html#child_process_subprocess_send_message_sendhandle_options_callback
     * @return {void}
     */
    send(...args: any[]): void;
    /**
     * Disconnects the child process. It will exit on its own
     * @return {void}
     */
    disconnect(): void;
    /**
     * Kills the child process
     *  https://nodejs.org/api/child_process.html#child_process_subprocess_kill_signal
     * @return {void}
     */
    kill(...args: any[]): void;
}
//# sourceMappingURL=processLauncher.d.ts.map