"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.ProcessManager = void 0;

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/trim"));

var _endsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/ends-with"));

var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/includes"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));

var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/reduce"));

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/json/stringify"));

var _embarkI18n = require("embark-i18n");

const ProcessState = {
  Stopped: 'stopped',
  Starting: 'starting',
  Running: 'running',
  Stopping: 'stopping',
  Errored: 'errored'
};

class ProcessManager {
  constructor(options) {
    this.logger = options.logger;
    this.events = options.events;
    this.plugins = options.plugins;
    this.processes = {};
    this.servicesState = {};
    this.plugin = this.plugins.createPlugin('processManager', {});
    this.events.on("servicesState", servicesState => {
      this.servicesState = servicesState;
    });

    this._registerApiCalls();

    this._registerEvents(); // TODO: removed because Process Manager shouldn't care or have knoweldge about deployment
    // this.events.once("deploy:beforeAll", this._registerCommands.bind(this));

  }

  _registerApiCalls() {
    this.plugin.registerAPICall('get', '/embark-api/services', (req, res) => {
      res.send(this._servicesForApi(this.servicesState));
    });
    this.plugin.registerAPICall('ws', '/embark-api/services', (ws, _res) => {
      this.events.on('servicesState', servicesState => {
        ws.send((0, _stringify.default)(this._servicesForApi(servicesState)), () => undefined);
      });
    });
    this.plugin.registerAPICall('get', '/embark-api/processes', (req, res) => {
      var _context;

      const formatter = (acc, processName) => {
        acc.push({
          state: this.processes[processName].state,
          name: processName
        });
        return acc;
      };

      res.send((0, _reduce.default)(_context = (0, _keys.default)(this.processes)).call(_context, formatter, []));
    });
  }

  _servicesForApi(servicesState) {
    let processList = [];

    for (let serviceName in servicesState) {
      let service = servicesState[serviceName];
      processList.push({
        state: service.status,
        name: serviceName,
        description: service.name
      });
    }

    return processList;
  }

  _registerCommands() {
    var _context2;

    // do not allow whisper service to be started/stopped as it requires a restart of embark
    const availableProcesses = (0, _filter.default)(_context2 = (0, _keys.default)(this.processes)).call(_context2, name => {
      var _context3;

      return !(0, _includes.default)(_context3 = ["whisper", "embark"]).call(_context3, name.toLowerCase());
    });
    this.plugin.registerConsoleCommand({
      description: (0, _embarkI18n.__)(`Starts/stops the process. Options: ${availableProcesses.join(", ")}`),
      matches: cmd => {
        return (0, _some.default)(availableProcesses).call(availableProcesses, name => {
          var _context4;

          name = name.toLowerCase();
          return (0, _includes.default)(_context4 = [`service ${name} on`, `service ${name} off`]).call(_context4, cmd.toLowerCase());
        });
      },
      usage: `service [process] on/off`,
      process: (cmd, callback) => {
        var _context5;

        const enable = (0, _endsWith.default)(_context5 = (0, _trim.default)(cmd).call(cmd)).call(_context5, 'on');
        const matches = cmd.match(/^service[\s](.*)[\s](?:on|off)$/) || [];
        const name = matches[1];
        this.logger.info(`${enable ? 'Starting' : 'Stopping'} the ${name} process...`);

        if (enable) {
          return this.events.request("processes:launch", name, (...args) => {
            const err = args[0];

            if (err) {
              this.logger.error(err); // writes to embark's console

              return callback(err); // passes message back to cockpit
            }

            const process = this.processes[name];

            if (process && process.afterLaunchFn) {
              process.afterLaunchFn.apply(process.afterLaunchFn, args);
            }

            callback(err, `${name} process started.`); // passes a message back to cockpit console
          });
        }

        this.events.request("processes:stop", name, err => {
          if (err) {
            this.logger.error(err); // writes to embark's console

            callback(err); // passes message back to cockpit
          }

          callback(err, `${name} process stopped.`); // passes a message back to cockpit console
        });
      }
    });
  }

  _registerEvents() {
    const self = this;
    self.events.setCommandHandler('processes:register', (name, cb) => {
      let launchFn, stopFn;

      if (typeof cb === 'object') {
        launchFn = cb.launchFn;
        stopFn = cb.stopFn;
      }

      this.processes[name] = {
        name: name,
        state: ProcessState.Stopped,
        cb: launchFn || cb,
        stopFn: stopFn || function noop() {}
      };
    });
    self.events.setCommandHandler('processes:launch', (name, cb) => {
      var _context6;

      cb = cb || function noop() {};

      let process = self.processes[name];

      if (!(0, _includes.default)(_context6 = [ProcessState.Stopped, ProcessState.Errored]).call(_context6, process.state)) {
        return cb((0, _embarkI18n.__)(`The ${name} process is already ${process.state.toLowerCase()}.`));
      }

      process.state = ProcessState.Starting;
      if (!process.afterLaunchFn) process.afterLaunchFn = cb;
      process.cb.apply(process.cb, [(...args) => {
        if (args[0]) {
          process.state = ProcessState.Errored;
        } else process.state = ProcessState.Running;

        cb.apply(cb, args);
      }]);
    });
    self.events.setCommandHandler('processes:stop', (name, cb) => {
      var _context7;

      let process = self.processes[name];

      if (!process) {
        // Process was never started
        return cb();
      }

      cb = cb || function noop() {};

      if (!(0, _includes.default)(_context7 = [ProcessState.Running, ProcessState.Errored]).call(_context7, process.state)) {
        return cb((0, _embarkI18n.__)(`The ${name} process is already ${process.state.toLowerCase()}.`));
      }

      process.state = ProcessState.Stopping;
      process.stopFn.apply(process.stopFn, [(...args) => {
        if (args[0]) {
          process.state = ProcessState.Errored;
        } else process.state = ProcessState.Stopped;

        cb.apply(cb, args);
      }]);
    });
  }

}

exports.ProcessManager = ProcessManager;
//# sourceMappingURL=processManager.js.map