"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.ProcessWrapper = void 0;

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));

var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var _setInterval2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/set-interval"));

var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));

var _fsExtra = require("fs-extra");

var _path = require("path");

var _eventsWrapper = require("./eventsWrapper");

const constants = (0, _fsExtra.readJsonSync)((0, _path.join)(__dirname, '../../constants.json'));

class ProcessWrapper {
  /**
   * Class from which process extend. Should not be instantiated alone.
   * Manages the log interception so that all console.* get sent back to the parent process
   * Also creates an Events instance. To use it, just do `this.events.[on|request]`
   *
   * @param {Options}     options    pingParent: true by default
   */
  constructor(options = {}) {
    process.on('uncaughtException', function (e) {
      process.send({
        error: e.stack
      });
    });
    this.options = (0, _assign.default)({
      pingParent: true
    }, options);
    this.interceptLogs();
    this.events = new _eventsWrapper.Events();

    if (this.options.pingParent) {
      this.pingParent();
    }
  } // Ping parent to see if it is still alive. Otherwise, let's die


  pingParent() {
    const self = this;
    self.retries = 0;

    function error() {
      if (self.retries > 2) {
        self.kill();
        process.exit(1);
      }

      self.retries++;
    }

    (0, _setInterval2.default)(() => {
      try {
        let result = self.send({
          action: 'ping'
        });

        if (!result) {
          return error();
        }

        self.retries = 0;
      } catch (e) {
        error();
      }
    }, 500);
  } // TODO: find better way - andre


  interceptLogs() {
    var _context, _context2, _context3, _context4, _context5, _context6, _context7;

    const context = {};
    context.console = console;
    context.console.log = (0, _bind.default)(_context = this._log).call(_context, this, 'log');
    context.console.warn = (0, _bind.default)(_context2 = this._log).call(_context2, this, 'warn');
    context.console.error = (0, _bind.default)(_context3 = this._log).call(_context3, this, 'error');
    context.console.info = (0, _bind.default)(_context4 = this._log).call(_context4, this, 'info');
    context.console.debug = (0, _bind.default)(_context5 = this._log).call(_context5, this, 'debug');
    context.console.trace = (0, _bind.default)(_context6 = this._log).call(_context6, this, 'trace');
    context.console.dir = (0, _bind.default)(_context7 = this._log).call(_context7, this, 'dir');
  }

  _log(type, ...messages) {
    const isHardSource = (0, _some.default)(messages).call(messages, message => {
      return typeof message === 'string' && (0, _indexOf.default)(message).call(message, 'hardsource') > -1;
    });

    if (isHardSource) {
      return;
    }

    this.send({
      result: constants.process.log,
      message: messages,
      type
    });
  }

  send() {
    if (!process.connected) {
      return false;
    }

    return process.send(...arguments);
  }

  kill() {
    // Should be implemented by derived class
    console.log('Process killed');
  }

}

exports.ProcessWrapper = ProcessWrapper;
//# sourceMappingURL=processWrapper.js.map