"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.ServicesMonitor = void 0;

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _setInterval2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/set-interval"));

var _embarkI18n = require("embark-i18n");

class ServicesMonitor {
  constructor(options) {
    this.events = void 0;
    this.logger = void 0;
    this.plugins = void 0;
    this.checkList = {};
    this.checkTimers = {};
    this.checkState = {};
    this.working = false;
    this.events = options.events;
    this.logger = options.logger;
    this.plugins = options.plugins;
    this.events.setCommandHandler("services:register", (checkName, checkFn, time, initialStatus) => {
      this.addCheck(checkName, checkFn, time, initialStatus);
    });
  }

  initCheck(checkName) {
    const self = this;
    const check = this.checkList[checkName];

    if (!check) {
      return false;
    }

    self.events.on('check:' + checkName, obj => {
      if (check && check.status === 'off' && obj.status === 'on') {
        self.events.emit('check:backOnline:' + checkName);
      }

      if (check && check.status === 'on' && obj.status === 'off') {
        self.events.emit('check:wentOffline:' + checkName);
      }

      check.status = obj.status; // const newState = {name: obj.name, status: obj.status, serviceName: checkName};
      // if (!deepEqual(newState, self.checkState[checkName])) {

      self.checkState[checkName] = {
        name: obj.name,
        status: obj.status,
        serviceName: checkName
      };
      self.events.emit("servicesState", self.checkState); // }
    });

    if (check.interval !== 0) {
      self.checkTimers[checkName] = (0, _setInterval2.default)(() => {
        check.fn.call(check.fn, obj => {
          self.events.emit('check:' + checkName, obj);
        });
      }, check.interval);
    }

    check.fn.call(check.fn, obj => {
      self.events.emit('check:' + checkName, obj);
    });
  }

  addCheck(checkName, checkFn, time, initialState) {
    this.logger.trace('add check: ' + checkName);
    this.checkList[checkName] = {
      fn: checkFn,
      interval: time || 5000,
      status: initialState
    };

    if (this.working) {
      this.initCheck(checkName);
    }
  }

  stopCheck(name) {
    clearInterval(this.checkTimers[name]);
    delete this.checkTimers[name];
    delete this.checkList[name];
    delete this.checkState[name];
  }

  startMonitor() {
    var _context;

    const self = this;
    this.working = true;
    this.logger.trace('startMonitor');
    const servicePlugins = this.plugins.getPluginsProperty('serviceChecks', 'serviceChecks');
    (0, _forEach.default)(servicePlugins).call(servicePlugins, pluginCheck => {
      self.addCheck(pluginCheck.checkName, pluginCheck.checkFn, pluginCheck.time);
    });
    (0, _forEach.default)(_context = (0, _keys.default)(this.checkList)).call(_context, checkName => {
      try {
        self.initCheck(checkName);
      } catch (err) {
        self.logger.error((0, _embarkI18n.__)("error running service check"));
        self.logger.error(err.message);
      }
    });
  }

}

exports.ServicesMonitor = ServicesMonitor;
//# sourceMappingURL=services_monitor.js.map